/*  Name:  mosdata.sas
   Title:  Assorted contingency table data sets for mosaic displays
*/

/*
  Running this program creates a SAS/IML storage catalog named
  MOSAIC.MOSDATA.  It is assumed that the libref MOSAIC has been
  defined, e.g., as follows:
libname mosaic   'c:\sasuser\mosaics';         *-- Windows ;
*/
*libname mosaic '~/sasuser/mosaics/v9';            *-- Unix    ;

/*
  To use one with mosaics,

  proc iml;
     reset storage=mosaic.mosaic;
     load module=_all_;
     reset storage=mosaic.mosdata;
     load module=bartlett;
     run bartlett;
     ...
*/
proc iml;

start bartlett;
/*
desc = '
In an experiment to investigate the effect of cuitting length (two levels)
and planting time (two levels) on the survival of plum root cuttings, 240
cuttings were planted for each of the 2 x 2 combinations of these factors,
and their survival was later recorded.  Bartlett used these data to illustrate
a method for testing for no three-way interaction in a contingency table.
';
*/

   data='bartlett';
   dim = {2 2 2};
   title="Bartlett data on plum root cuttings";
	source='Bartlett:35,Hand-etal:94 #19';
   table = {156 84 84 156,
            107 133 31 209};
   vnames= {'Alive?' 'Time' 'Length' ,
            'A' 'T' 'L'};
   vnames = vnames[1,];
   lnames= {'Alive' 'Dead',
            'Now' 'Spring',
            'Long' 'Short'};
   finish;

start berkeley;
/*
desc = '
Aggregate data on applicants to graduate school at Berkeley for the six
largest departments in 1973 classified by admission and sex.
';
*/
   data='berkeley';
   title='Berkeley Admissions Data';
	source='Bickel-etal:75';
   dim = {2 2 6};
   vnames = {"Admit" "Gender" "Dept"};
   lnames = {"Admitted" "Rejected" " "  " "  " "  " ",
             "Male"  "Female" " "  " "  " "  " ",
             "A" "B" "C" "D" "E" "F"};
          /* Admit Not */
   table = { 512  313,
              89   19,
             353  207,
              17    8,
             120  205,
             202  391,
             138  279,
             131  244,
              53  138,
              94  299,
              22  351,
              24  317};
   *  print table;
   finish;

start cancer;
   data='cancer';
/*
desc = '
Three year survival of 474 breast cancer patients according to nuclear
grade and diagnostic centre.
';
        Data from Morrison etal
   Whittaker, J. (1990) Graphical Models, p. 220.
   Lindsey, J.K. (1995) Modelling Frequency and Count Data, p38
*/
	source='Morrison etal, Lindsey:95 (p38)';
   dim= {2 2 2};
   table = {
   /*
      Malignant   benign
      Died Surv Died Surv
   */
      35   59   47   112,   /* Boston    */
      42   77   26    76};  /* Glamorgan */

   vnames = {'Survival' 'Grade' 'Center'};
   lnames = {'Died' 'Surv',
            'Malignant' 'Benign',
            'Boston' 'Glamorgan'};
   title = 'Survival of Breast Cancer Patients';
   finish;

start caesar;
/*
desc = '
Data from infection from birth by Caesarian section,
classified by Risk (two levels), whether antibiotics
were used (two levels) and whether the Caesarian section
was planned or not.  The outcome is infection (three levels).
';

See also:
   Fahrmeir package (caesar)
   
*/
   data='caesar';
   source='Fahrmeir:94';
	title = 'Risk factors for infection in caesarian births';
   dim = {3 2 2 2};
   vnames = {'Infection' 'Risk?' 'Antibiotics' 'Planned'};
   lnames = {'Type 1' 'Type 2' 'None',
            'Yes' 'No' '',
            'Yes' 'No' '',
            'Yes' 'No' ''
            };
   table = {
      0   1  17,
      0   1   1,
      11  17  30,
      4   4  32,
      4   7  87,
      0   0   0,
      10  13   3,
      0   0   9
            };
   finish;

start detergen;
/*
desc='
Cross-classification of a sample of 1008 consumers according to 
(a) the softness of the laundry water used, (b) previous use of
detergent Brand M, (c) the termperature of laundry water used
and (d) expressed preference for Brand X or Brand M in a blind
trial.
';
*/

   data='detergen';
   source='Fienberg:80 (p. 71), RiesSmith:63';
	title = 'Detergent preference data';
   dim = {2 2 2 3};
   vnames = {'Temperature' 'M-User?' 'Preference' 'Water softness'};
   lnames = {'High' 'Low' '',
            'Yes'  'No'  '',
            'Brand X' 'Brand M' '',
            'Soft' 'Medium' 'Hard'};

   table = {
      19  57  29  63,  /* Soft     Brand X */
      29  49  27  53,  /* Soft     Brand M */
      23  47  33  66,  /* Medium   Brand X */
      47  55  23  50,  /* Medium   Brand M */
      24  37  42  68,  /* Hard     Brand X */
      43  52  30  42   /* Hard     Brand M */
   };
   finish;

start dyke;
	data='dyke';
	source='DykePatterson:52, Fienberg:77 (Table 5-6)';
/*
desc = '
Bovervational data on a sample of 1729 individuals, cross-
classified in a 2^5 table according to their sources of
information (read newspapers, listen to the radio, do
`solid' reading, attend lectures) and whether they have
good or poor knowledge regarding cancer.  Knowledge of cancer
is sometimes treated as the resonse.
';

!Sources of knowledge of cancer Dyke & Patterson (1952)
!(Fienberg, 1977, Table 5-6)
!lindsey/cat3ex/ch2ex8.dat
!Stokes, Davis, Koch, sec 14.4
!         Y                     N            Radio
!    Y         N           Y         N      Reading
!Good Poor Good Poor   Good Poor Good Poor
!                                           Newspaper   Lectures
*/
   table = {
  23    8    8    4     27   18    7    6,  /*   Y          Y */
 102   67   35   59    201  177   75  156,  /*   Y          N */
   1    3    4    3      3    8    2   10,  /*   N          Y */
  16   16   13   50     67   83   84  393   /*   N          N */
   };
   dim = {2 2 2 2 2};
   vnames = {'Knowledge' 'Reading' 'Radio' 'Lectures' 'Newspaper'};
   lnames = {'Good' 'Poor',
            'Yes' 'No',
            'Yes' 'No',
            'Yes' 'No',
            'Yes' 'No' };

	*-- Reversing order of Yes/No vars makes nicer displays;
	table = table[{4 3 2 1},];
	table = table[,{7 8 5 6 3 4 1 2}];
   table = shape(table, 16, 2);
	lnames[2,] = lnames[2,{2 1}];
	lnames[3,] = lnames[3,{2 1}];
	lnames[4,] = lnames[4,{2 1}];
	lnames[5,] = lnames[5,{2 1}];

   title='Sources of knowledge of cancer';
   source='Fienberg:77 Table 5-6';
   finish;

start employ;
/*
desc = '
Employment status on Jan. 1 1975, by cause of layoff and length of
previous employment at time of layoff for employees who lost their
job in Fall 1974 in Denmark.
';
        "In 1974 the Danish National Inst for Social Science Research
        investigated 1314 employees who left their jobs during the 2nd
        half of the year.
        Classified by:
                Employment status, 1/1/75:  New job vs. still unemployed
                Cause of layoff:            Closure, etc. vs. Replacement
                Length of employment at time of layoff
*/

   data='employ';
   title  = 'Employment Status Data';
   source = 'Andersen (1991), Ex 5.3 (p.167); from Kjer (1978) Table 4.8';
   dim = {2 2 6};
   vnames = {'EmployStatus' 'Layoff' 'LengthEmploy'};
   lnames = {'NewJob' 'Unemployed'    '' '' '' '',
             'Closure'  'Replaced' '' '' '' '',
             '<1 Mo' '1-3 Mo' '3-12 Mo' '1-2 Yr' '2-5 Yr' '>5 Yr'};

       /* B: -Closure-   -Replaced-                 */
       /* A: Job  Unem   Job  Unem                  */
   table = {   8   10,     40   24,        /* < 1 Mo */
              35   42,     85   42,        /* 1-3 Mo */
              70   86,    181   41,        /* 3-12Mo */
              62   80,     85   16,        /* 1-2 Yr */
              56   67,    118   27,        /* 2-5 Yr */
              38   35,     56   10 };      /* > 5 Yr */
	finish;

start gilby;
   title = 'Clothing and intelligence rating of children';
   source = 'Gilby & Pearson 1911, from Anscombe 1981, p 302';
   /*
   Gilby, W. H. and Pearson, K.
   On the significance of the teacher's appreciation of
   general intelligence.  Biometrika, 8, 93-108 (esp p 94)
   Quoted by Kendall (1943,...1953) Table 13.1, p 320

desc = '
   Schoolboys were classified according to their clothing
   and to their teachers rating of 'dullness'
   Note: the last two categories of clothing were pooled
   as were the first two categories of dullness.
';
   */
   data='gilby';
   dim = {6 4};
   vnames = {'Dullness' 'Clothing'};
   lnames = {
      'Mentally defective or Slow dull' 'Slow' 'Slow intelligent' 'Fairly intelligent' 'Distinctly capable' 'Very able',
      'Very well clad' 'Well clad' 'Poor but passable' 'Insufficient or worse' '' ''
      };
	*-- Shorter labels for the categories;
   lnames = {
      'Ment. defective' 'Slow' 'Slow Intell' 'Fairly Intell' 'Capable' 'V.Able',
      'V.Well clad' 'Well clad' 'Passable' 'Insufficient' '' ''
      };
   table = {
         33  48 113 209 194  39,
         41 100 202 255 138  15,
         39  58  70  61  33   4,
         17  13  22  10  10   1 };
   finish;

start haireye;
   *-- Hair color, eye color data;
        data='haireye';
        source = 'Snee (1974)';
        table = {
 /* ----brown---   -----blue-----   ----hazel---   ---green--- */
   32  38  10  3   11  50  10  30   10  25  7  5   3  15  7  8,   /* M */
   36  81  16  4    9  34   7  64    5  29  7  5   2  14  7  8 }; /* F */
** changed table[,2] from {38,81} **;
 table = {
 /* ----brown---   -----blue-----   ----hazel---   ---green--- */
   32  53  10  3   11  50  10  30   10  25  7  5   3  15  7  8,   /* M */
   36  66  16  4    9  34   7  64    5  29  7  5   2  14  7  8 }; /* F */


   dim    = { 4 4 2 };
   vnames = {'Hair' 'Eye' 'Sex' };          /* Variable names */
   lnames = {                               /* Category names */
         'Black' 'Brown' 'Red' 'Blond',    /* hair color */
         'Brown' 'Blue' 'Hazel' 'Green',   /* eye color  */
         'Male' 'Female' ' '  ' ' };       /* sex        */
   title  = 'Hair color - Eye color data';
   finish;

start heart;
   data='heart';
   source='Karger, 1980';
	title = 'Sex, occupation and heart disease';

   dim = {2 2 3};
   vnames = {'Disease' 'Gender' 'Occup'};
   lnames = {'Disease' 'None'    '',
             'Male'   'Female'   '',
             'Unempl' 'WhiteCol' 'BlueCol'};
	table = {
	254     759,   /*  Male    Unempl  */
	431     10283, /*  Female  Unempl  */
	158     3155,  /*  Male    WhiteCol*/        
	52      3082,  /*  Female  WhiteCol*/        
	87      2829,  /*  Male    BlueCol */
	16      416};  /*  Female  BlueCol */
	finish;

start heckman;
   data='heckman';
	source='HeckmanWillis:77,Lindsey:93 (p. 185)';
/*
From Lindsey, cat3dat/ch9e5.dat
see also Lindsey 93, Table 6.2, p185
!Labour force participation of married women 1967-1971
!Heckman, J.J. & Willis, R.J. (1977) "A beta-logistic model for the analysis
!of sequential labor force participation by married women." Jr. Pol. Econ.
!85: 27-58
!1971    1970 1969 1968 1967
!Yes No
*/
   table = {
      426  38 , /*Yes  Yes  Yes  Yes */
         16  47 , /*No   Yes  Yes  Yes */
       11   2 , /*Yes  No   Yes  Yes */
       12  28 , /*No   No   Yes  Yes */
       21   7 , /*Yes  Yes  No   Yes */
        0   9 , /*No   Yes  No   Yes */
        8   3 , /*Yes  No   No   Yes */
        5  43 , /*No   No   No   Yes */
       73  11 , /*Yes  Yes  Yes  No  */
        7  17 , /*No   Yes  Yes  No  */
        9   3 , /*Yes  No   Yes  No  */
        5  24 , /*No   No   Yes  No  */
       54  16 , /*Yes  Yes  No   No  */
        6  28 , /*No   Yes  No   No  */
       36  24 , /*Yes  No   No   No  */
       35 559}; /*No   No   No   No  */

	title='Labour force participation of married women 1967-1971';
   dim = {2 2 2 2 2};
   vnames = '19' + ('71':'67');
   lnames = repeat({'Working   ' 'Not Working'}, 5, 1);
   lnames = repeat({'Yes  ' 'No   '}, 5, 1);
   lnames[,1] = ('71':'67')` + lnames[,1];
   finish;

start hoyt;
/*
desc = '
Minnesota high school graduates of June 1930 were classified with
respect to (a) position by thirds in their graduating class,
(b) post-high school status in April 1939 (4 levels), (c) sex,
(d) father's occupational status (7 levels, from 1=High to 7=Low)
';
*/

        source='Hoyt, Krishnaiah, Torrence (1959); Fienberg pp.91-92';
        data='hoyt';
        title = 'Minnesota High School Graduates';
   dim   = { 4 3 7 2};
   vnames= {'Status' 'Rank' 'Occupation' 'Sex'};
   lnames= {'College' 'School' 'Job' 'Other' '' '' '',
            'Low' 'Middle' 'High' '' '' '' '',
            '1' '2' '3' '4' '5' '6' '7',
            'Male' 'Female' '' '' '' '' ''};
   table = {
     87   3  17 105  216   4  14 118  256   2  10  53 ,
     72   6  18 209  159  14  28 227  176   8  22  95 ,
     52  17  14 541  119  13  44 578  119  10  33 257 ,
     88   9  14 328  158  15  36 304  144  12  20 115 ,
     32   1  12 124   43   5   7 119   42   2   7  56 ,
     14   2   5 148   24   6  15 131   24   2   4  61 ,
     20   3   4 109   41   5  13  88   32   2   4  41 ,
     53   7  13  76  163  30  28 118  309  17  38  89 ,
     36  16  11 111  116  41  53 214  225  49  68 210 ,
     52  28  49 521  162  64 129 708  243  79 184 448 ,
     48  18  29 191  130  47  62 305  237  57  63 219 ,
     12   5  10 101   35  11  37 152   72  20  21  95 ,
      9   1  15 130   19  13  22 174   42  10  19 105 ,
      3   1   6  88   25   9  15 158   36  14  19  93 };
   finish;

start marital;
        source='Agresti, Table 7.3';
        data='marital';
  *-- define the data variables;
                    /* Gender Pre  Extra */
  table={ 17   4 ,  /* Women  Yes  Yes  */
          54  25 ,  /* Women  Yes  No   */
          36   4 ,  /* Women  No   Yes  */
         214 322 ,  /* Women  No   No   */
          28  11 ,  /* Men    Yes  Yes  */
          60  42 ,  /* Men    Yes  No   */
          17   4 ,  /* Men    No   Yes  */
          68 130 }; /* Men    No   No   */
   dim = { 2 2 2 2 };
   vnames = {'Marital' 'Extra' 'Pre' 'Gender'};
   lnames = {'Divorced'   'Married',
             'Extra Sex: Yes' 'No',
             'Pre Sex: Yes'   'No',
             'Women    '      'Men' };
   title  = 'Pre/Extramarital Sex and Marital Status';
   finish;

start mental;
	data = 'mental';
	title='Mental impariment and parents SES';
	source='Haberman, 1979 [p.375], from Srole etal,(1978) p.289';
	* also, Agresti:90, Lindsey p.99;	
	table = {
		64   94   58   46
		57   94   54   40
		57  105   65   60
		72  141   77   94
		36   97   54   78
		21   71   54   71
		};
   dim = { 4 6 };
   vnames= {"Mental Impairment" "Parents SES"};
   lnames= {"Well" "Mild" "Moderate" "Impaired" " " " ",
            "High" "2" "3" "4" 5 "Low"};
	finish;

start mobility;
   data='mobility';
	source='FeathermanHauser:78';
   dim = { 5 5 };
   /* Social Mobility data, Featherman & Hauser, 78 ,
            analyzed by Falguerolles & Mathieu, COMPSTAT 88 */
   /*    Sons occupation
         UNM   LNM    UM    LM  Farm */
   table = {
        1414   521   302   643    40,
         724   524   254   703    48,
         798   648   856  1676   108,
         756   914   771  3325   237,
         409   357   441  1611  1832 };

   title  = {'Social Mobility data'};
   vnames = {"Son's Occupation" "Father's Occupation"};
   lnames = { 'UpNonMan' 'LoNonMan' 'UpManual' 'LoManual' 'Farm',
            'UpNonMan' 'LoNonMan' 'UpManual' 'LoManual'  'Farm'};
   finish;

start abortion;
   data='abortion';
   title='Abortion opinion data';
   source='Christiensen (\S 3.5.2) ';  *-- page 92;

   dim = {2 2 2};
/* SES:      Low      NotLow  */
/* Opinion: Y   N     Y   N   */
   table ={171  79   138 112,      /* Female */
           152 148   167 133};     /* Male   */

   run marg(loc,newtab,dim,table,(3:1)`);
   table = shape(newtab,2,4);
   vnames={'Sex' 'Status' 'Support Abortion'};
   lnames={'Female' 'Male',
           'Lo'  'Hi',
           'Yes' 'No'};
   finish;

start suicide;
   data = 'suicide';
	source = 'Heuer:79,Friendly:94a';
   table = {
     512    25   852    64   875    52   477    29   229     3,  /* Gun */
     335    40   883   113   625    91   201    45    45    29,  /* Gas */
    1524   212  2751   575  3936  1481  3581  2014  2948  1355,  /* Hang */
    1160   921  2823  1672  2465  2224  1531  2283   938  1548,  /* Poison */
     189   131   366   276   244   327   273   388   268   383,  /* Jump */
      67    30   213   139   247   354   207   679   212   501   /* Drown */
            };
   dim = {2 5 6};

   meth= {'Gun' 'Gas' 'Hang' 'Poison' 'Jump' 'Drown'};
   age =  char(do(10,55,15),2,0)+shape({'-'},1,4)+
          char(do(20,65,15),2,0)
           || {'>65'  ' '   };
   sex = {'Male' 'Female' ' '  ' '  ' '  ' '  };

   vnames = {'Sex' 'Age' 'Method'};
   lnames = (sex // age // meth) ;
   title  = 'Suicide data';
   free meth age sex;
   finish;

start titanic;
   data='titanic';
	source='Dawson:95';
   title='Survival on the Titanic';
   dim = {4 2 2 2};
   vnames = {'Class' 'Sex' 'Age' 'Survived'};
   lnames = {'1st' '2nd' '3rd' 'Crew',
            'Male'  'Female' '' '',
            'Child' 'Adult' '' '',
            'Died'  'Survived'   '' ''};
   table = {
  /*
 CLASS1    CLASS2    CLASS3    CLASS4    SURVIVE     AGE      SEX
  */
    0         0        35         0,   /* No       Child    Male   */
    0         0        17         0,   /* No       Child    Female */
  118       154       387       670,   /* No       Adult    Male   */
    4        13        89         3,   /* No       Adult    Female */
    5        11        13         0,   /* Yes      Child    Male   */
    1        13        14         0,   /* Yes      Child    Female */
   57        14        75       192,   /* Yes      Adult    Male   */
  140        80        76        20    /* Yes      Adult    Female */
        };
   table = table[{3 4 1 2  7 8 5 6},];
   lnames[3,] = {'Adult' 'Child' '' ''};
   finish;

start victims;
	data='victims';
	source = 'Reiss:80,Fienberg:80 (Table 2-8)';
   crime = {'Rape' 'Assault' 'Robbery' 'PickPock' 'Pers.Larceny'
            'Burglary' 'Hous.Larceny' 'Auto Theft'};
   dim = {8 8};
   vnames = {'First Victimization' 'Second Victimization'};
   lnames = crime // crime ;
   title  = 'Repeat Victimization Data';
   table = {  26   50  11   6    82   39   48   11,
              65 2997 238  85  2553 1083 1349  216,
              12  279 197  36   459  197  221   47,
               3  102  40  61   243  115  101   38,
              75 2628 413 329 12137 2658 3689  687,
              52 1117 191 102  2649 3210 1973  301,
              42 1251 206 117  3757 1962 4646  391,
               3  221  51  24   678  301  367  269}`;
   free crime;
   finish;

/*  transform a vector of character strings into a matrix, whose
    number of rows is ncol(dim) and number of columns is max(dim).
	 Useful for creating lnames.
	e.g.,
		ln = {a1 a2 a3 b1 b2 b3 b4 c1 c2};
		lname = vec2mat({3 4 2}, ln);
*/
start vec2mat(dim, vec);
   r = ncol(dim);
   c = max(dim);
   len = max(length(vec));
   blank='                    ';
   mat = j(r, c, substr(blank, 1, len));
   start = 1;
   do i = 1 to r;
      l = dim[i];
      mat[i, 1:l] = shape(vec[start:start+l-1],1);
      start = start+l;
      end;
   return(mat);
   finish;

/*
*-- Stack two matrices, filling out columns of the smaller with 0s or ' 's;
start ontop(mat1, mat2);
   if type(mat1) = 'N'
      then fill=0;
      else fill=' ';
   nc1 = ncol(mat1);
   nc2 = ncol(mat2);
   if ncol(mat1) = ncol(mat2)
      then return (mat1 // mat2);
   else if nc1 < nc2 then do;
      result = (mat1 || j( nrow(mat1), nc2-nc1, fill)) // mat2;
      end;
   else do;
      result = mat1 // (mat2 || j( nrow(mat2), nc1-nc2, fill));
      end;
   return(result);
   finish;
*/

start datalist(datasets) global(data, title, source, dim, vnames, lnames);
   file print;

   put 'dataname' @12 'title / dim / vnames' /;
   do i = 1 to ncol(datasets);
		title = '????';
      call execute('run ', datasets[i], ';');
		namelist = namelist // data;
		titles   = titles // title;
      d = rowcat(char(dim,1+length(vnames[1])));
      v = rowcat(vnames+' ');
      put data @12 title / @12 'dim: ' d / @12 'vn:    ' v;
      put;
      end;
		
   finish;


   datasets = {
    abortion bartlett  berkeley  cancer  caesar  detergen  dyke
	 employ  gilby haireye  heart heckman hoyt  marital mental mobility
	 suicide titanic victims};

   reset storage=mosaic.mosdata;
   store module=_all_ datasets;
   show storage;

	*-- Make sure all are mentioned in datasets;
   stored = storage();
   util = {vec2mat, datalist, datasets};
   dif = t(setdif(stored, datasets));
   dif = setdif(dif, util);
   if type(dif)^='U' then print stored dif;

   run datalist(datasets);
quit;         ;
